from typing import Any, Tuple, TypedDict, Literal, Optional
import numpy as np
from PIL import Image
from pathlib import Path

dataset_dir = Path(__file__).resolve().parent.parent.parent / "dataset"


class ConverstationText(TypedDict):
    type: Literal["text"]
    text: str


class ConverstationAudio(TypedDict):
    type: Literal["audio"]
    audio_url: str


class ConverstationImage(TypedDict):
    type: Literal["image"]
    image_url: str


class Conversation(TypedDict):

    role: Literal["user", "assistant", "system"]
    content: list[ConverstationText | ConverstationAudio | ConverstationImage]


class DatasetOutput(TypedDict):
    chat: list[Conversation]
    answer: Optional[str]
    original: Any
    images: Optional[list[Image.Image]]
    audios: Optional[list[Tuple[np.ndarray, int]]]
