import pytest
from .factory import get_dataset, MAPPING_NAME_TO_DATASET

# Get all registered dataset names for parameterization
dataset_names = list(MAPPING_NAME_TO_DATASET.keys())


@pytest.mark.parametrize("dataset_name", dataset_names)
def test_registered_datasets(dataset_name):
    dataset = get_dataset(dataset_name)

    # Test train split
    assert "train" in dataset, f"Train split not found in {dataset_name}"
    assert len(dataset["train"]) > 0, f"Train split is empty for {dataset_name}"  # type: ignore
    assert "chat" in dataset["train"][0], f"'chat' key not found in first train sample of {dataset_name}"  # type: ignore
    assert len(dataset["train"][0]["chat"]) > 0, f"'chat' is empty in first train sample of {dataset_name}"  # type: ignore

    # Test test split
    assert "test" in dataset, f"Test split not found in {dataset_name}"
    assert len(dataset["test"]) > 0, f"Test split is empty for {dataset_name}"  # type: ignore
    assert "chat" in dataset["test"][0], f"'chat' key not found in first test sample of {dataset_name}"  # type: ignore
    assert len(dataset["test"][0]["chat"]) > 0, f"'chat' is empty in first test sample of {dataset_name}"  # type: ignore
